@@ -87,6 +87,11 @@ static void gnome_ditem_edit_changed      (GnomeDItemEdit      *dee);
 static void gnome_ditem_edit_icon_changed (GnomeDItemEdit      *dee);
 static void gnome_ditem_edit_name_changed (GnomeDItemEdit      *dee);
 
+static void set_relation (GtkWidget *widget, GtkLabel *label);
+static void destroy_tooltip (GtkObject *object);
+static void set_tooltip (GnomeDItemEdit *dee, GtkWidget *widget,
+			 const gchar *description);
+
 enum {
         CHANGED,
         ICON_CHANGED,
@@ -287,6 +292,8 @@ fill_easy_page (GnomeDItemEdit *dee,
 				 dee, G_CONNECT_SWAPPED);
 	dee->_priv->name_entry = entry;
 
+	set_relation (dee->_priv->name_entry, GTK_LABEL (label));
+
 	label = label_new (_("Comment:"));
 	table_attach_label (GTK_TABLE (table), label, 0, 1, 1, 2);
 
@@ -298,6 +305,8 @@ fill_easy_page (GnomeDItemEdit *dee,
                                  dee, G_CONNECT_SWAPPED);
 	dee->_priv->comment_entry = entry;
 
+	set_relation (dee->_priv->comment_entry, GTK_LABEL (label));
+
 	label = label_new (_("Command:"));
 	table_attach_label (GTK_TABLE (table), label, 0, 1, 2, 3);
 	dee->_priv->exec_label = label;
@@ -310,6 +319,8 @@ fill_easy_page (GnomeDItemEdit *dee,
 				 dee, G_CONNECT_SWAPPED);
 	dee->_priv->exec_entry = entry;
 
+	set_relation (dee->_priv->exec_entry, GTK_LABEL (label));
+
 	label = label_new (_("Type:"));
 	table_attach_label (GTK_TABLE (table), label, 0, 1, 3, 4);
 	dee->_priv->type_label = label;
@@ -328,6 +339,8 @@ fill_easy_page (GnomeDItemEdit *dee,
 				 dee, G_CONNECT_SWAPPED);
 	dee->_priv->type_combo = combo;
 
+	set_relation (dee->_priv->type_combo, GTK_LABEL (label));
+
 	label = label_new (_("Icon:"));
 	table_attach_label (GTK_TABLE (table), label, 0, 1, 4, 5);
 
@@ -346,6 +359,8 @@ fill_easy_page (GnomeDItemEdit *dee,
 		GTK_BOX (hbox), icon_entry, FALSE, FALSE, 0);
         dee->_priv->icon_entry = icon_entry;
 
+	set_relation (dee->_priv->icon_entry, GTK_LABEL (label));
+
 	align = gtk_alignment_new (0.0, 0.5, 0.0, 0.0);
 	gtk_box_pack_start (GTK_BOX (hbox), align, FALSE, FALSE, 0);
 
@@ -587,6 +602,8 @@ fill_advanced_page (GnomeDItemEdit *dee,
 				  G_CALLBACK (gnome_ditem_edit_changed), dee);
 	dee->_priv->tryexec_entry = entry;
 
+	set_relation (dee->_priv->tryexec_entry, GTK_LABEL (label));
+
 	label = label_new (_("Documentation:"));
 	table_attach_label (GTK_TABLE (page), label, 0, 1, 1, 2);
 
@@ -597,6 +614,8 @@ fill_advanced_page (GnomeDItemEdit *dee,
 				  G_CALLBACK (gnome_ditem_edit_changed), dee);
 	dee->_priv->doc_entry = entry;
 
+	set_relation (dee->_priv->doc_entry, GTK_LABEL (label));
+
 	label = gtk_label_new (_("Name/Comment translations:"));
 	table_attach_label (GTK_TABLE (page), label, 0, 2, 2, 3);
 
@@ -608,26 +627,39 @@ fill_advanced_page (GnomeDItemEdit *dee,
 	gtk_widget_set_size_request (entry, 30, -1);
 	dee->_priv->transl_lang_entry = entry;
 
+	set_tooltip (dee, GTK_WIDGET(entry), _("Language"));
+	set_relation (dee->_priv->transl_lang_entry, GTK_LABEL (label));
+
 	entry = gtk_entry_new ();
 	gtk_box_pack_start (GTK_BOX (box), entry, TRUE, TRUE, 0);
 	dee->_priv->transl_name_entry = entry;
 
+	set_tooltip (dee, GTK_WIDGET(entry), _("Name"));
+	set_relation (dee->_priv->transl_name_entry, GTK_LABEL (label));
+
 	entry = gtk_entry_new ();
 	gtk_box_pack_start (GTK_BOX (box), entry, TRUE, TRUE, 0);
 	dee->_priv->transl_comment_entry = entry;
 
+	set_tooltip (dee, GTK_WIDGET(entry), _("Comment"));
+	set_relation (dee->_priv->transl_comment_entry, GTK_LABEL (label));
+
 	box = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
 	table_attach_entry (GTK_TABLE (page), box, 0, 2, 4, 5);
-  
 	button = gtk_button_new_with_label (_("Add/Set"));
 	gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 0);
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (translations_add), dee);
+  
+	set_tooltip (dee, GTK_WIDGET(button),
+		    _("Add or Set Name/Comment Translations"));
 
 	button = gtk_button_new_with_label (_("Remove"));
 	gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 0);
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (translations_remove), dee);
+	set_tooltip (dee, GTK_WIDGET(button),
+		    _("Remove Name/Comment Translation"));
   
 	dee->_priv->translations = setup_translations_list (dee);
 
@@ -719,6 +751,8 @@ gnome_ditem_edit_destroy (GtkObject *object)
 		gnome_desktop_item_unref (de->_priv->ditem);
 	de->_priv->ditem = NULL; /* just for sanity */
 
+	destroy_tooltip (object);
+
 	GNOME_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
 }
 
@@ -1239,3 +1273,75 @@ gnome_ditem_edit_set_directory_only (GnomeDItemEdit *dee,
 		}
 	}
 }
+
+/**
+ * set_relation
+ * @widget : The Gtk widget which is labelled by @label
+ * @label : The label for the @widget.
+ * Description : This function establishes atk relation
+ * between a gtk widget and a label.
+ */
+static void
+set_relation (GtkWidget *widget, GtkLabel *label)
+{
+	AtkObject *aobject;
+	AtkRelationSet *relation_set;
+	AtkRelation *relation;
+	AtkObject *targets[1];
+
+	g_return_if_fail (GTK_IS_WIDGET(widget));
+	g_return_if_fail (GTK_IS_LABEL(label));
+
+	aobject = gtk_widget_get_accessible (widget);
+
+	/* Return if GAIL is not loaded */
+	if (! GTK_IS_ACCESSIBLE (aobject))
+		return;
+
+	/* Set the ATK_RELATION_LABEL_FOR relation */
+	gtk_label_set_mnemonic_widget (label, widget);
+
+	targets[0] = gtk_widget_get_accessible (GTK_WIDGET (label));
+
+	relation_set = atk_object_ref_relation_set (aobject);
+
+	relation = atk_relation_new (targets, 1, ATK_RELATION_LABELLED_BY);
+	atk_relation_set_add (relation_set, relation);
+	g_object_unref (G_OBJECT (relation));
+}
+
+static void
+destroy_tooltip (GtkObject *object)
+{
+	GtkTooltips *tooltips;
+
+	tooltips = g_object_get_data (G_OBJECT (object), "tooltips");
+	if (tooltips) {
+		g_object_unref (tooltips);
+		g_object_set_data (G_OBJECT (object), "tooltips", NULL);
+	}
+}
+
+/**
+ * set_tooltip
+ * Description : Set @description as the tooltip for the @widget.
+ *		 Creates the GtkTooltips structure if not already present.
+ */
+static void
+set_tooltip (GnomeDItemEdit *dee, GtkWidget *widget, const gchar *description)
+{
+        GtkTooltips *tooltips;
+
+	tooltips = g_object_get_data (G_OBJECT (dee), "tooltips");
+
+	/* create if not already present */
+	if (!tooltips) {
+		tooltips = gtk_tooltips_new ();
+		g_return_if_fail (tooltips != NULL);
+		g_object_ref (tooltips);
+		gtk_object_sink (GTK_OBJECT (tooltips));
+		g_object_set_data (G_OBJECT (dee), "tooltips", tooltips);
+	}
+
+        gtk_tooltips_set_tip (tooltips, widget, description, NULL);
+}
